"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getProofTypeFromKey = exports.getSphereonDidResolver = exports.getSphereonSuppliedSignatureFromJwtIssuer = exports.getSupportedJwaSignatureAlgorithms = void 0;
const core_1 = require("@credo-ts/core");
/**
 * Returns the JWA Signature Algorithms that are supported by the wallet.
 *
 * This is an approximation based on the supported key types of the wallet.
 * This is not 100% correct as a supporting a key type does not mean you support
 * all the algorithms for that key type. However, this needs refactoring of the wallet
 * that is planned for the 0.5.0 release.
 */
function getSupportedJwaSignatureAlgorithms(agentContext) {
    const supportedKeyTypes = agentContext.wallet.supportedKeyTypes;
    // Extract the supported JWS algs based on the key types the wallet support.
    const supportedJwaSignatureAlgorithms = supportedKeyTypes
        // Map the supported key types to the supported JWK class
        .map(core_1.getJwkClassFromKeyType)
        // Filter out the undefined values
        .filter((jwkClass) => jwkClass !== undefined)
        // Extract the supported JWA signature algorithms from the JWK class
        .flatMap((jwkClass) => jwkClass.supportedSignatureAlgorithms);
    return supportedJwaSignatureAlgorithms;
}
exports.getSupportedJwaSignatureAlgorithms = getSupportedJwaSignatureAlgorithms;
async function getSphereonSuppliedSignatureFromJwtIssuer(agentContext, jwtIssuer) {
    var _a;
    let key;
    let alg;
    let kid;
    let did;
    if (jwtIssuer.method === 'did') {
        const didsApi = agentContext.dependencyManager.resolve(core_1.DidsApi);
        const didDocument = await didsApi.resolveDidDocument(jwtIssuer.didUrl);
        const verificationMethod = didDocument.dereferenceKey(jwtIssuer.didUrl, ['authentication']);
        // get the key from the verification method and use the first supported signature algorithm
        key = (0, core_1.getKeyFromVerificationMethod)(verificationMethod);
        const _alg = (_a = (0, core_1.getJwkClassFromKeyType)(key.keyType)) === null || _a === void 0 ? void 0 : _a.supportedSignatureAlgorithms[0];
        if (!_alg)
            throw new core_1.CredoError(`No supported signature algorithms for key type: ${key.keyType}`);
        alg = _alg;
        kid = verificationMethod.id;
        did = verificationMethod.controller;
    }
    else {
        throw new core_1.CredoError(`Unsupported jwt issuer method '${jwtIssuer.method}'. Only 'did' is supported.`);
    }
    return {
        signature: async (data) => {
            if (typeof data !== 'string')
                throw new core_1.CredoError("Expected string but received 'Uint8Array'");
            const signedData = await agentContext.wallet.sign({
                data: core_1.TypedArrayEncoder.fromString(data),
                key,
            });
            const signature = core_1.TypedArrayEncoder.toBase64URL(signedData);
            return signature;
        },
        alg: alg,
        did,
        kid,
    };
}
exports.getSphereonSuppliedSignatureFromJwtIssuer = getSphereonSuppliedSignatureFromJwtIssuer;
function getSphereonDidResolver(agentContext) {
    return {
        resolve: async (didUrl) => {
            var _a;
            const didsApi = agentContext.dependencyManager.resolve(core_1.DidsApi);
            const result = await didsApi.resolve(didUrl);
            return Object.assign(Object.assign({}, result), { didDocument: (_a = result.didDocument) === null || _a === void 0 ? void 0 : _a.toJSON() });
        },
    };
}
exports.getSphereonDidResolver = getSphereonDidResolver;
function getProofTypeFromKey(agentContext, key) {
    const signatureSuiteRegistry = agentContext.dependencyManager.resolve(core_1.SignatureSuiteRegistry);
    const supportedSignatureSuites = signatureSuiteRegistry.getAllByKeyType(key.keyType);
    if (supportedSignatureSuites.length === 0) {
        throw new core_1.CredoError(`Couldn't find a supported signature suite for the given key type '${key.keyType}'.`);
    }
    return supportedSignatureSuites[0].proofType;
}
exports.getProofTypeFromKey = getProofTypeFromKey;
//# sourceMappingURL=utils.js.map